@@ -187,7 +187,6 @@ normalize_idle_time(unsigned int idle_time)
  * MCAST_DEFAULT_MAX entries. */
 struct mcast_snooping *
 mcast_snooping_create(void)
-    OVS_NO_THREAD_SAFETY_ANALYSIS
 {
     struct mcast_snooping *ms;
 
@@ -322,9 +321,7 @@ mcast_group_insert_bundle(struct mcast_snooping *ms OVS_UNUSED,
 /* Return true if multicast still has bundles associated.
  * Return false if there is no bundles. */
 static bool
-mcast_group_has_bundles(const struct mcast_snooping *ms OVS_UNUSED,
-                        const struct mcast_group *grp)
-    OVS_REQ_RDLOCK(ms->rwlock)
+mcast_group_has_bundles(struct mcast_group *grp)
 {
     return !ovs_list_is_empty(&grp->bundle_lru);
 }
@@ -334,7 +331,6 @@ mcast_group_has_bundles(const struct mcast_snooping *ms OVS_UNUSED,
 static void
 mcast_snooping_flush_group__(struct mcast_snooping *ms,
                              struct mcast_group *grp)
-    OVS_REQ_WRLOCK(ms->rwlock)
 {
     ovs_assert(ovs_list_is_empty(&grp->bundle_lru));
     hmap_remove(&ms->table, &grp->hmap_node);
@@ -398,7 +394,7 @@ mcast_snooping_prune_expired(struct mcast_snooping *ms,
         expired++;
     }
 
-    if (!mcast_group_has_bundles(ms, grp)) {
+    if (!mcast_group_has_bundles(grp)) {
         mcast_snooping_flush_group__(ms, grp);
         expired++;
     }
@@ -547,14 +543,6 @@ mcast_snooping_add_mld(struct mcast_snooping *ms,
     offset += MLD_HEADER_LEN;
     addr = dp_packet_at(p, offset, sizeof(struct in6_addr));
 
-    if (!addr) {
-        /* We error out if the provided packet is not large enough to handle
-         * the mld->type values below.  Note that whenever a new type gets
-         * added, its data should be equal to or larger than the size of
-         * struct in6_addr. */
-        return 0;
-    }
-
     switch (mld->type) {
     case MLD_REPORT:
         ret = mcast_snooping_add_group(ms, addr, vlan, port,
@@ -665,7 +653,7 @@ mrouter_get_lru(const struct mcast_snooping *ms,
     OVS_REQ_RDLOCK(ms->rwlock)
 {
     if (!ovs_list_is_empty(&ms->mrouter_lru)) {
-        *m = mcast_mrouter_from_lru_node(ms->mrouter_lru.next);
+        *m = mcast_mrouter_from_lru_node(ovs_list_front(&ms->mrouter_lru));
         return true;
     } else {
         *m = NULL;
@@ -717,9 +705,7 @@ mcast_snooping_add_mrouter(struct mcast_snooping *ms, uint16_t vlan,
 }
 
 static void
-mcast_snooping_flush_mrouter(const struct mcast_snooping *ms OVS_UNUSED,
-                             struct mcast_mrouter_bundle *mrouter)
-    OVS_REQ_WRLOCK(ms->rwlock)
+mcast_snooping_flush_mrouter(struct mcast_mrouter_bundle *mrouter)
 {
     ovs_list_remove(&mrouter->mrouter_node);
     free(mrouter);
@@ -740,7 +726,7 @@ mcast_snooping_port_get(const struct ovs_list *list,
                         struct mcast_port_bundle **f)
 {
     if (!ovs_list_is_empty(list)) {
-        *f = mcast_port_from_list_node(list->next);
+        *f = mcast_port_from_list_node(ovs_list_front(list));
         return true;
     } else {
         *f = NULL;
@@ -832,7 +818,7 @@ mcast_snooping_mdb_flush__(struct mcast_snooping *ms)
     hmap_shrink(&ms->table);
 
     while (mrouter_get_lru(ms, &mrouter)) {
-        mcast_snooping_flush_mrouter(ms, mrouter);
+        mcast_snooping_flush_mrouter(mrouter);
     }
 }
 
@@ -865,7 +851,7 @@ mcast_snooping_flush__(struct mcast_snooping *ms)
 
     /* flush multicast routers */
     while (mrouter_get_lru(ms, &mrouter)) {
-        mcast_snooping_flush_mrouter(ms, mrouter);
+        mcast_snooping_flush_mrouter(mrouter);
     }
 
     /* flush flood ports */
@@ -915,7 +901,7 @@ mcast_snooping_run__(struct mcast_snooping *ms)
     mrouter_expired = 0;
     while (mrouter_get_lru(ms, &mrouter)
            && time_now() >= mrouter->expires) {
-        mcast_snooping_flush_mrouter(ms, mrouter);
+        mcast_snooping_flush_mrouter(mrouter);
         mrouter_expired++;
     }
 
@@ -1007,7 +993,7 @@ mcast_snooping_flush_bundle(struct mcast_snooping *ms, void *port)
         if (mcast_group_delete_bundle(ms, g, port)) {
             ms->need_revalidate = true;
 
-            if (!mcast_group_has_bundles(ms, g)) {
+            if (!mcast_group_has_bundles(g)) {
                 mcast_snooping_flush_group__(ms, g);
             }
         }
@@ -1015,7 +1001,7 @@ mcast_snooping_flush_bundle(struct mcast_snooping *ms, void *port)
 
     LIST_FOR_EACH_SAFE (m, mrouter_node, &ms->mrouter_lru) {
         if (m->port == port) {
-            mcast_snooping_flush_mrouter(ms, m);
+            mcast_snooping_flush_mrouter(m);
             ms->need_revalidate = true;
         }
     }
